@@ -291,7 +291,7 @@ mod test {
         ));
 
         let groups = s.group_tuples(false, true);
-        let aggregated = unsafe { s.agg_list(&groups) };
+        let aggregated = unsafe { s.agg_list(&groups?) };
         match aggregated.get(0) {
             AnyValue::List(s) => {
                 assert!(matches!(s.dtype(), DataType::Categorical(_)));
