@@ -12,6 +12,7 @@ pub struct PartitionGroupByExec {
     maintain_order: bool,
     slice: Option<(i64, usize)>,
     input_schema: SchemaRef,
+    from_partitioned_ds: bool,
 }
 
 impl PartitionGroupByExec {
@@ -22,6 +23,7 @@ impl PartitionGroupByExec {
         maintain_order: bool,
         slice: Option<(i64, usize)>,
         input_schema: SchemaRef,
+        from_partitioned_ds: bool,
     ) -> Self {
         Self {
             input,
@@ -30,6 +32,7 @@ impl PartitionGroupByExec {
             maintain_order,
             slice,
             input_schema,
+            from_partitioned_ds,
         }
     }
 
@@ -111,7 +114,6 @@ fn estimate_unique_count(keys: &[Series], mut sample_size: usize) -> PolarsResul
     // u: total unique groups counted in sample
     // ui: groups with single unique value counted in sample
     let set_size = keys[0].len();
-    let offset = (keys[0].len() / 2) as i64;
     if set_size < sample_size {
         sample_size = set_size;
     }
@@ -135,6 +137,7 @@ fn estimate_unique_count(keys: &[Series], mut sample_size: usize) -> PolarsResul
         let groups = s.group_tuples(true, false)?;
         Ok(finish(&groups))
     } else {
+        let offset = (keys[0].len() / 2) as i64;
         let keys = keys
             .iter()
             .map(|s| s.slice(offset, sample_size))
@@ -152,6 +155,7 @@ fn can_run_partitioned(
     keys: &[Series],
     original_df: &DataFrame,
     state: &ExecutionState,
+    from_partitioned_ds: bool,
 ) -> PolarsResult<bool> {
     if std::env::var("POLARS_NO_PARTITION").is_ok() {
         if state.verbose() {
@@ -194,7 +198,16 @@ fn can_run_partitioned(
             eprintln!("{} unique values: {}", sampled_method, unique_estimate);
         }
 
-        if unique_estimate > unique_count_boundary {
+        if from_partitioned_ds {
+            let estimated_cardinality = unique_estimate as f32 / original_df.height() as f32;
+            if estimated_cardinality < 0.4 {
+                eprintln!("PARTITIONED DS");
+                Ok(true)
+            } else {
+                eprintln!("PARTITIONED DS: estimated cardinality: {} exceeded the boundary: 0.4, running default HASH AGGREGATION", estimated_cardinality);
+                Ok(false)
+            }
+        } else if unique_estimate > unique_count_boundary {
             if state.verbose() {
                 eprintln!("estimated unique count: {} exceeded the boundary: {}, running default HASH AGGREGATION",unique_estimate, unique_count_boundary)
             }
@@ -218,7 +231,7 @@ impl PartitionGroupByExec {
             // of groups.
             let keys = self.keys(&original_df, state)?;
 
-            if !can_run_partitioned(&keys, &original_df, state)? {
+            if !can_run_partitioned(&keys, &original_df, state, self.from_partitioned_ds)? {
                 return groupby_helper(
                     original_df,
                     keys,
