@@ -1,8 +1,4 @@
-use polars_core::prelude::*;
-
-use crate::physical_plan::executors::evaluate_physical_expressions;
-use crate::physical_plan::state::ExecutionState;
-use crate::prelude::*;
+use super::*;
 
 /// Take an input Executor (creates the input DataFrame)
 /// and a multiple PhysicalExpressions (create the output Series)
@@ -15,15 +11,12 @@ pub struct ProjectionExec {
     pub(crate) schema: SchemaRef,
 }
 
-impl Executor for ProjectionExec {
-    fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
-        #[cfg(debug_assertions)]
-        {
-            if state.verbose() {
-                println!("run ProjectionExec")
-            }
-        }
-        let df = self.input.execute(state)?;
+impl ProjectionExec {
+    fn execute_impl(
+        &mut self,
+        state: &mut ExecutionState,
+        df: DataFrame,
+    ) -> PolarsResult<DataFrame> {
         state.set_schema(self.input_schema.clone());
 
         let df = evaluate_physical_expressions(&df, &self.expr, state, self.has_windows);
@@ -44,3 +37,34 @@ impl Executor for ProjectionExec {
         df
     }
 }
+
+impl Executor for ProjectionExec {
+    fn execute(&mut self, state: &mut ExecutionState) -> PolarsResult<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run ProjectionExec")
+            }
+        }
+        let df = self.input.execute(state)?;
+
+        let profile_name = if state.has_node_timer() {
+            let by = self
+                .expr
+                .iter()
+                .map(|s| Ok(s.to_field(&self.input_schema)?.name))
+                .collect::<PolarsResult<Vec<_>>>()?;
+            let name = column_delimited("projection".to_string(), &by);
+            Cow::Owned(name)
+        } else {
+            Cow::Borrowed("")
+        };
+
+        if state.has_node_timer() {
+            let new_state = state.clone();
+            new_state.record(|| self.execute_impl(state, df), profile_name)
+        } else {
+            self.execute_impl(state, df)
+        }
+    }
+}
