@@ -4,12 +4,146 @@ use super::super::executors::{self, Executor};
 use super::*;
 use crate::utils::*;
 
+fn partitionable_gb(
+    keys: &[Node],
+    aggs: &[Node],
+    _input_schema: &Schema,
+    expr_arena: &Arena<AExpr>,
+    apply: &Option<Arc<dyn DataFrameUdf>>,
+) -> bool {
+    // We first check if we can partition the groupby on the latest moment.
+    let mut partitionable = true;
+
+    // checks:
+    //      1. complex expressions in the groupby itself are also not partitionable
+    //          in this case anything more than col("foo")
+    //      2. a custom function cannot be partitioned
+    //      3. we don't bother with more than 2 keys, as the cardinality likely explodes
+    //         by the combinations
+    if !keys.is_empty() && keys.len() < 3 && apply.is_none() {
+        // complex expressions in the groupby itself are also not partitionable
+        // in this case anything more than col("foo")
+        for key in keys {
+            if (expr_arena).iter(*key).count() > 1 {
+                partitionable = false;
+                break;
+            }
+        }
+
+        if partitionable {
+            for agg in aggs {
+                let aexpr = expr_arena.get(*agg);
+                let depth = (expr_arena).iter(*agg).count();
+
+                if matches!(aexpr, AExpr::Count) {
+                    continue;
+                }
+                // col()
+                // lit() etc.
+                if depth == 1 {
+                    partitionable = false;
+                    break;
+                }
+
+                // it should end with an aggregation
+                if let AExpr::Alias(input, _) = aexpr {
+                    // col().agg().alias() is allowed: count of 3
+                    // col().alias() is not allowed: count of 2
+                    // count().alias() is allowed: count of 2
+                    if depth <= 2 {
+                        match expr_arena.get(*input) {
+                            AExpr::Count => {}
+                            _ => {
+                                partitionable = false;
+                                break;
+                            }
+                        }
+                    }
+                }
+
+                let has_aggregation =
+                    |node: Node| has_aexpr(node, expr_arena, |ae| matches!(ae, AExpr::Agg(_)));
+
+                // check if the aggregation type is partitionable
+                // only simple aggregation like col().sum
+                // that can be divided in to the aggregation of their partitions are allowed
+                if !((expr_arena).iter(*agg).all(|(_, ae)| {
+                    use AExpr::*;
+                    match ae {
+                        // struct is needed to keep both states
+                        #[cfg(feature = "dtype-struct")]
+                        Agg(AAggExpr::Mean(_)) => {
+                            // only numeric means for now.
+                            // logical types seem to break because of casts to float.
+                            matches!(expr_arena.get(*agg).get_type(_input_schema, Context::Default, expr_arena).map(|dt| {
+                                        dt.is_numeric()}), Ok(true))
+                        },
+                        // only allowed expressions
+                        Agg(agg_e) => {
+                            matches!(
+                                            agg_e,
+                                            AAggExpr::Min{..}
+                                                | AAggExpr::Max{..}
+                                                | AAggExpr::Sum(_)
+                                                | AAggExpr::Last(_)
+                                                | AAggExpr::First(_)
+                                                | AAggExpr::Count(_)
+                                        )
+                        },
+                        Function {input, options, ..} => {
+                            matches!(options.collect_groups, ApplyOptions::ApplyFlat) && input.len() == 1 &&
+                                !has_aggregation(input[0])
+                        }
+                        BinaryExpr {left, right, ..} => {
+                            !has_aggregation(*left) && !has_aggregation(*right)
+                        }
+                        Ternary {truthy, falsy, predicate,..} => {
+                            !has_aggregation(*truthy) && !has_aggregation(*falsy) && !has_aggregation(*predicate)
+                        }
+                        Column(_) | Alias(_, _) | Count | Literal(_) | Cast {..} => {
+                            true
+                        }
+                        _ => {
+                            false
+                        },
+                    }
+                }) &&
+                    // we only allow expressions that end with an aggregation
+                    matches!(aexpr, AExpr::Alias(_, _) | AExpr::Agg(_)))
+                {
+                    partitionable = false;
+                    break;
+                }
+
+                #[cfg(feature = "object")]
+                {
+                    for name in aexpr_to_leaf_names(*agg, expr_arena) {
+                        let dtype = _input_schema.get(&name).unwrap();
+
+                        if let DataType::Object(_) = dtype {
+                            partitionable = false;
+                            break;
+                        }
+                    }
+                    if !partitionable {
+                        break;
+                    }
+                }
+            }
+        }
+    } else {
+        partitionable = false;
+    }
+    partitionable
+}
+
 pub fn create_physical_plan(
     root: Node,
     lp_arena: &mut Arena<ALogicalPlan>,
     expr_arena: &mut Arena<AExpr>,
 ) -> PolarsResult<Box<dyn Executor>> {
     use ALogicalPlan::*;
+
     let logical_plan = lp_arena.take(root);
     match logical_plan {
         #[cfg(feature = "python")]
@@ -195,15 +329,13 @@ pub fn create_physical_plan(
             options,
         } => {
             let input_schema = lp_arena.get(input).schema(lp_arena).into_owned();
-            let input = create_physical_plan(input, lp_arena, expr_arena)?;
-
             let phys_keys = create_physical_expressions(&keys, Context::Default, expr_arena)?;
-
             let phys_aggs = create_physical_expressions(&aggs, Context::Aggregation, expr_arena)?;
 
             let _slice = options.slice;
             #[cfg(feature = "dynamic_groupby")]
             if let Some(options) = options.dynamic {
+                let input = create_physical_plan(input, lp_arena, expr_arena)?;
                 return Ok(Box::new(executors::GroupByDynamicExec {
                     input,
                     keys: phys_keys,
@@ -216,6 +348,7 @@ pub fn create_physical_plan(
 
             #[cfg(feature = "dynamic_groupby")]
             if let Some(options) = options.rolling {
+                let input = create_physical_plan(input, lp_arena, expr_arena)?;
                 return Ok(Box::new(executors::GroupByRollingExec {
                     input,
                     keys: phys_keys,
@@ -227,127 +360,16 @@ pub fn create_physical_plan(
             }
 
             // We first check if we can partition the groupby on the latest moment.
-            let mut partitionable = true;
-
-            // checks:
-            //      1. complex expressions in the groupby itself are also not partitionable
-            //          in this case anything more than col("foo")
-            //      2. a custom function cannot be partitioned
-            //      3. we don't bother with more than 2 keys, as the cardinality likely explodes
-            //         by the combinations
-            if !keys.is_empty() && keys.len() < 3 && apply.is_none() {
-                // complex expressions in the groupby itself are also not partitionable
-                // in this case anything more than col("foo")
-                for key in keys {
-                    if (&*expr_arena).iter(key).count() > 1 {
-                        partitionable = false;
-                        break;
-                    }
-                }
-
-                if partitionable {
-                    for agg in &aggs {
-                        let aexpr = expr_arena.get(*agg);
-                        let depth = (&*expr_arena).iter(*agg).count();
-
-                        // col()
-                        // lit() etc.
-                        if depth == 1 {
-                            partitionable = false;
-                            break;
-                        }
-
-                        // it should end with an aggregation
-                        if let AExpr::Alias(input, _) = aexpr {
-                            // col().agg().alias() is allowed: count of 3
-                            // col().alias() is not allowed: count of 2
-                            // count().alias() is allowed: count of 2
-                            if depth <= 2 {
-                                match expr_arena.get(*input) {
-                                    AExpr::Count => {}
-                                    _ => {
-                                        partitionable = false;
-                                        break;
-                                    }
-                                }
-                            }
-                        }
-
-                        let has_aggregation = |node: Node| {
-                            has_aexpr(node, expr_arena, |ae| matches!(ae, AExpr::Agg(_)))
-                        };
-
-                        // check if the aggregation type is partitionable
-                        // only simple aggregation like col().sum
-                        // that can be divided in to the aggregation of their partitions are allowed
-                        if !((&*expr_arena).iter(*agg).all(|(_, ae)| {
-                            use AExpr::*;
-                            match ae {
-                                // struct is needed to keep both states
-                                #[cfg(feature = "dtype-struct")]
-                                Agg(AAggExpr::Mean(_)) => {
-                                    // only numeric means for now.
-                                    // logical types seem to break because of casts to float.
-                                    matches!(expr_arena.get(*agg).get_type(&input_schema, Context::Default, expr_arena).map(|dt| {
-                                        dt.is_numeric()}), Ok(true))
-                                },
-                                // only allowed expressions
-                                Agg(agg_e) => {
-                                    matches!(
-                                            agg_e,
-                                            AAggExpr::Min{..}
-                                                | AAggExpr::Max{..}
-                                                | AAggExpr::Sum(_)
-                                                | AAggExpr::Last(_)
-                                                | AAggExpr::First(_)
-                                                | AAggExpr::Count(_)
-                                        )
-                                },
-                                Function {input, options, ..} => {
-                                    matches!(options.collect_groups, ApplyOptions::ApplyFlat) && input.len() == 1 &&
-                                        !has_aggregation(input[0])
-                                }
-                                BinaryExpr {left, right, ..} => {
-                                    !has_aggregation(*left) && !has_aggregation(*right)
-                                }
-                                Ternary {truthy, falsy, predicate,..} => {
-                                    !has_aggregation(*truthy) && !has_aggregation(*falsy) && !has_aggregation(*predicate)
-                                }
-                                Column(_) | Alias(_, _) | Count | Literal(_) | Cast {..} => {
-                                    true
-                                }
-                                _ => {
-                                    false
-                                },
-                            }
-                        }) &&
-                            // we only allow expressions that end with an aggregation
-                            matches!(aexpr, AExpr::Alias(_, _) | AExpr::Agg(_)))
-                        {
-                            partitionable = false;
-                            break;
-                        }
-
-                        #[cfg(feature = "object")]
-                        {
-                            for name in aexpr_to_leaf_names(*agg, expr_arena) {
-                                let dtype = input_schema.get(&name).unwrap();
-
-                                if let DataType::Object(_) = dtype {
-                                    partitionable = false;
-                                    break;
-                                }
-                            }
-                            if !partitionable {
-                                break;
-                            }
-                        }
-                    }
-                }
-            } else {
-                partitionable = false;
-            }
+            let partitionable = partitionable_gb(&keys, &aggs, &input_schema, expr_arena, &apply);
             if partitionable {
+                let from_partitioned_ds = (&*lp_arena).iter(input).any(|(_, lp)| {
+                    if let ALogicalPlan::Union { options, .. } = lp {
+                        options.from_partitioned_ds
+                    } else {
+                        false
+                    }
+                });
+                let input = create_physical_plan(input, lp_arena, expr_arena)?;
                 Ok(Box::new(executors::PartitionGroupByExec::new(
                     input,
                     phys_keys,
@@ -355,8 +377,10 @@ pub fn create_physical_plan(
                     maintain_order,
                     options.slice,
                     input_schema,
+                    from_partitioned_ds,
                 )))
             } else {
+                let input = create_physical_plan(input, lp_arena, expr_arena)?;
                 Ok(Box::new(executors::GroupByExec::new(
                     input,
                     phys_keys,
