@@ -1,7 +1,9 @@
 pub(crate) mod aggregates;
 mod generic;
 mod primitive;
+mod string;
 mod utils;
 
 pub(crate) use generic::*;
 pub(crate) use primitive::*;
+pub(crate) use string::*;
