@@ -177,6 +177,13 @@ where
                         )) as Box<dyn Sink>
                     })
                 }
+                (DataType::Utf8, 1) => Box::new(groupby::Utf8GroupbySink::new(
+                    key_columns[0].clone(),
+                    aggregation_columns,
+                    agg_fns,
+                    output_schema.clone(),
+                    options.slice,
+                )) as Box<dyn Sink>,
                 _ => Box::new(groupby::GenericGroupbySink::new(
                     key_columns,
                     aggregation_columns,
@@ -214,9 +221,10 @@ where
             };
             Box::new(op) as Box<dyn Operator>
         }
-        HStack { exprs, .. } => {
+        HStack { exprs, schema, .. } => {
             let op = operators::HstackOperator {
                 exprs: exprs_to_physical(exprs, expr_arena, &to_physical)?,
+                input_schema: schema.clone(),
             };
             Box::new(op) as Box<dyn Operator>
         }
