@@ -467,11 +467,9 @@ pub fn datetime(args: DatetimeArgs) -> Expr {
                 if let (Some(y), Some(m), Some(d), Some(h), Some(mnt), Some(s), Some(us)) =
                     (y, m, d, h, mnt, s, us)
                 {
-                    Some(
-                        NaiveDate::from_ymd(y, m, d)
-                            .and_hms_micro(h, mnt, s, us)
-                            .timestamp_micros(),
-                    )
+                    NaiveDate::from_ymd_opt(y, m, d)
+                        .and_then(|nd| nd.and_hms_micro_opt(h, mnt, s, us))
+                        .map(|ndt| ndt.timestamp_micros())
                 } else {
                     None
                 }
@@ -899,51 +897,26 @@ pub fn sum_exprs<E: AsRef<[Expr]>>(exprs: E) -> Expr {
 /// Get the the maximum value per row
 pub fn max_exprs<E: AsRef<[Expr]>>(exprs: E) -> Expr {
     let exprs = exprs.as_ref().to_vec();
-    max_exprs_impl(exprs)
-}
-
-fn max_exprs_impl(mut exprs: Vec<Expr>) -> Expr {
-    if exprs.len() == 1 {
-        return std::mem::take(&mut exprs[0]);
+    if exprs.is_empty() {
+        return Expr::Columns(Vec::new());
     }
-
-    let first = std::mem::take(&mut exprs[0]);
-    first
-        .map_many(
-            |s| {
-                let s = s.to_vec();
-                let df = DataFrame::new_no_checks(s);
-                df.hmax().map(|s| s.unwrap())
-            },
-            &exprs[1..],
-            GetOutput::super_type(),
-        )
-        .alias("max")
+    let func = |s1, s2| {
+        let df = DataFrame::new_no_checks(vec![s1, s2]);
+        df.hmax().map(|s| s.unwrap())
+    };
+    reduce_exprs(func, exprs).alias("max")
 }
 
-/// Get the the minimum value per row
 pub fn min_exprs<E: AsRef<[Expr]>>(exprs: E) -> Expr {
     let exprs = exprs.as_ref().to_vec();
-    min_exprs_impl(exprs)
-}
-
-fn min_exprs_impl(mut exprs: Vec<Expr>) -> Expr {
-    if exprs.len() == 1 {
-        return std::mem::take(&mut exprs[0]);
+    if exprs.is_empty() {
+        return Expr::Columns(Vec::new());
     }
-
-    let first = std::mem::take(&mut exprs[0]);
-    first
-        .map_many(
-            |s| {
-                let s = s.to_vec();
-                let df = DataFrame::new_no_checks(s);
-                df.hmin().map(|s| s.unwrap())
-            },
-            &exprs[1..],
-            GetOutput::super_type(),
-        )
-        .alias("min")
+    let func = |s1, s2| {
+        let df = DataFrame::new_no_checks(vec![s1, s2]);
+        df.hmin().map(|s| s.unwrap())
+    };
+    reduce_exprs(func, exprs).alias("min")
 }
 
 /// Evaluate all the expressions with a bitwise or
@@ -1062,6 +1035,7 @@ pub fn coalesce(exprs: &[Expr]) -> Expr {
         options: FunctionOptions {
             collect_groups: ApplyOptions::ApplyGroups,
             cast_to_supertypes: true,
+            input_wildcard_expansion: true,
             ..Default::default()
         },
     }
