@@ -1,7 +1,7 @@
 use polars::prelude::*;
 
 #[test]
-fn test_to_list_logical() -> Result<()> {
+fn test_to_list_logical() -> PolarsResult<()> {
     let ca = Utf8Chunked::new("a", &["2021-01-01", "2021-01-02", "2021-01-03"]);
     let out = ca.as_date(None)?.into_series();
     let out = out.to_list().unwrap();
