@@ -2093,7 +2093,8 @@ def arg_where(
 def coalesce(
     exprs: Sequence[
         pli.Expr | str | date | datetime | timedelta | int | float | bool | pli.Series
-    ],
+    ]
+    | pli.Expr,
 ) -> pli.Expr:
     """
     Folds the expressions from left to right, keeping the first non-null value.
